<?php

function rvm_get_config_file_name()
{
  global $rvm_config;
  unset($rvm_config);
  return dirname(__FILE__).'/../_rvgm_config.dat';
}

function rvm_get_config()
{
  global $rvm_config;
  if (isset($rvm_config))
    return $rvm_config;

  $x = @file_get_contents( rvm_get_config_file_name() );
  if ($x!==false)
    $rvm_config = unserialize($x);
  if ( !is_array(@$rvm_config) )
  {
    $rvm_config = array(
        'nb_markers' => 25,
      );
  }
  return $rvm_config;
}

function rvm_parse_kml_url($tokens, &$next_token)
{
  $page=array();

  $page = array_merge($page, parse_section_url($tokens, $next_token) );
  if ( !isset($page['section']) )
    $page['section'] = 'categories';
    
  $page = array_merge( $page, parse_well_known_params_url( $tokens, $next_token) );
  $page['start']=0;
  $page['box'] = rvm_bounds_from_url( @$_GET['box'] );
  return $page;
}

function rvm_duplicate_blowup_url($redefined=array(), $removed=array())
{
  return rvm_make_blowup_url(
    params_for_duplication($redefined, $removed)
    );
}

function rvm_make_blowup_url($params)
{
  global $conf, $rvm_dir;
  if ( file_exists(PHPWG_ROOT_PATH.'mapl.php') )
    $url = get_root_url().'mapl';
  else
    $url = get_root_url().'plugins/'.$rvm_dir.'/mapl';
  if ($conf['php_extension_in_urls'])
    $url .= '.php';
  if ($conf['question_mark_in_urls'])
    $url .= '?';
  $url .= make_section_in_url($params);
  $url = add_well_known_params_in_url($url, array_intersect_key($params, array('flat'=>1) ) );
  
  $get_params = array();
  if ( isset($params['box']) and !empty($params['box']) and !bounds_is_world($params['box']) )
    $get_params['box'] = bounds_to_url($params['box']);
  if ( isset($params['start']) and $params['start']>0 )
    $get_params['start'] = $params['start'];
  $url = add_url_params($url, $get_params );
  return $url;
}

function rvm_parse_blowup_url($tokens, &$next_token)
{
  $page=array();
  $page = array_merge($page, parse_section_url($tokens, $next_token) );
  if ( !isset($page['section']) )
    $page['section'] = 'categories';
  $page = array_merge( $page, parse_well_known_params_url( $tokens, $next_token) );
  $page['start']=0;
  if ( isset($_GET['start']) )
    $page['start']=$_GET['start'];
  $page['box'] = rvm_bounds_from_url( @$_GET['box'] );
  return $page;
}

function rvm_bounds_from_url($str)
{
  if ( !isset($str) or strlen($str)==0 )
    return null;
  $r = explode(',', $str );
  if ( count($r) != 4)
    bad_request( $str.' is not a valid geographical bound' );
  $b = array(
      's' => $r[0],
      'w' => $r[1],
      'n' => $r[2],
      'e' => $r[3],
    );
  return $b;
}

function rvm_bounds_to_sql( $b )
{
  if ( !isset($b) or empty($b) )
    return 'i.lat IS NOT NULL';
  $sql_where = 'i.lat BETWEEN '.$b['s'].' AND '.($b['n']);
  $sql_where .= ' AND ';
  if ($b['e'] >= $b['w'])
    $sql_where .= 'i.lon BETWEEN '.$b['w'].' AND '.($b['e']);
  else
    $sql_where .= 'i.lon NOT BETWEEN '.($b['w']+1e-7).' AND '.($b['e']-1e-7);
  return $sql_where;
}



function bounds_to_url($b, $p = 6 )
{
  if ( empty($b) )
    return '';
  return round($b['s'],$p).','.round($b['w'],$p).','.round($b['n'],$p).','.round($b['e'],$p);
}

function bounds_is_world( $b )
{
  if (empty($b)) return false;
  return $b['n']>=90 and $b['s']<=-90 and $b['w']<=-180 and $b['e']>=180;
}

function bounds_add($bounds, $lat, $lon)
{
  if ( empty($bounds) )
  {
    $bounds = array(
      's' => $lat,
      'n' => $lat,
      'w' => $lon,
      'e' => $lon,
      'alat' => $lat,
      'alon' => $lon,
      'count' => 1
      );
  }
  else
  {
    if ( $lat>$bounds['n'] )
      $bounds['n']=$lat;
    elseif ( $lat<$bounds['s'] )
      $bounds['s']=$lat;
    if ( $lon>$bounds['e'] )
      $bounds['e']=$lon;
    elseif ( $lon<$bounds['w'] )
      $bounds['w']=$lon;
      
    $bounds['alat'] = ($bounds['alat']*$bounds['count'] + $lat ) / ($bounds['count']+1);
    $bounds['alon'] = ($bounds['alon']*$bounds['count'] + $lon ) / ($bounds['count']+1);
    $bounds['count']=$bounds['count']+1;
  }

  return $bounds;
}

function bounds_union($b1, $b2)
{
  $total_count = $b1['count'] + $b2['count'];
  return
    array_merge(
      $b1,
      array(
        's' => min( $b1['s'], $b2['s'] ),
        'n' => max( $b1['n'], $b2['n'] ),
        'w' => min( $b1['w'], $b2['w'] ),
        'e' => max( $b1['e'], $b2['e'] ),
        'alat' => ($b1['alat']*$b1['count'] + $b2['alat']*$b2['count'])/$total_count,
        'alon' => ($b1['alon']*$b1['count'] + $b2['alon']*$b2['count'])/$total_count,
        'count' => $total_count,
        'min_date' => min( $b1['min_date'], $b2['max_date'] ),
        'max_date' => max( $b1['max_date'], $b2['max_date'] ),
      )
    );
}

function bounds_center($b)
{
  if (empty($b))
    return array();
  return array(
    'lat' => ($b['s']+$b['n'])/2,
    'lon' => ($b['w']+$b['e'])/2,
    );
}

function bounds_sw($b)
{
  if (empty($b))
    return array();
  return array(
    'lat' => $b['s'],
    'lon' => $b['w'],
    );
}

function bounds_ne($b)
{
  if (empty($b))
    return array();
  return array(
    'lat' => $b['n'],
    'lon' => $b['e'],
    );
}


function bounds_lat_range($b)
{
  if (empty($b))
    return 0;
  return $b['n'] - $b['s'];
}

function bounds_lon_range($b)
{
  if (empty($b))
    return 0;
  return $b['e'] - $b['w'];
}


function rvm_get_cat_bounds($forbidden)
{
  $cache_file_name = rvm_get_cache_file_name();
  if ( file_exists($cache_file_name) )
  {
    return unserialize( file_get_contents($cache_file_name) );
  }
  
  $query = '
SELECT category_id cat_id, uppercats, MAX(lat) AS n, MIN(lat) AS s, MAX(lon) AS e, MIN(lon) AS w, AVG(lat) AS alat, AVG(lon) AS alon, COUNT(i.id) count, MIN(i.date_creation) min_date, MAX(i.date_creation) max_date
  FROM 
    '.CATEGORIES_TABLE.' as c
      INNER JOIN 
    '.IMAGE_CATEGORY_TABLE.' ON category_id = c.id
      INNER JOIN 
    '.IMAGES_TABLE.' i ON image_id=i.id
  WHERE lat IS NOT NULL '.$forbidden.'
  GROUP BY category_id
;';
  //echo "<pre>" . var_export($query,true);

  $result = pwg_query($query);
  $uppercats_list = array();
  $cat_bounds = array();
  while ($row = mysql_fetch_assoc($result))
  {
    array_push($uppercats_list, $row['uppercats']);
    $cat_id = $row['cat_id'];
    unset( $row['cat_id'], $row['uppercats'] );
    $cat_bounds[ $cat_id ] = $row;
    $cat_bounds[ $cat_id ]['self'] = $row;
  }
  natsort($uppercats_list);
  //echo "<pre>" . var_export($uppercats_list,true);

  foreach ( array_reverse($uppercats_list) as $uppercats)
  {
    $cat_ids = explode(',', $uppercats);
    $bounds = $cat_bounds[ $cat_ids[count($cat_ids)-1] ];
    for ($i=count($cat_ids)-2; $i>=0; $i--)
    {
      $this_bounds = @$cat_bounds[ $cat_ids[$i] ];
      if ( !isset($this_bounds) )
      {
        $this_bounds = $bounds;
        unset($this_bounds['self']);
      }
      else
      {
//        if ($cat_ids[$i]==43) echo "<pre>Before\n" . var_export($this_bounds,true);
//        if ($cat_ids[$i]==43) echo "<pre>Union\n" . var_export($bounds,true);
        $this_bounds = bounds_union($this_bounds, $bounds);
      }
      $this_bounds['nb_cats'] = 1 + @$this_bounds['nb_cats'];
//      if ($cat_ids[$i]==43) echo "<pre>" . var_export($this_bounds,true);
      $cat_bounds[ $cat_ids[$i] ] = $this_bounds;
    }
  }
  //echo "<pre>" . var_export($cat_bounds,true);

  $file = fopen($cache_file_name , 'w' );
  fwrite($file, serialize($cat_bounds) );
  fclose( $file );
  
  return $cat_bounds;
}


function marray_from_query($query)
{
  $ret = array();
  $result = pwg_query($query);
  while ($row = mysql_fetch_assoc($result))
    $ret[] = $row;
  return $ret;
}

function build_section_items($img_fields, $where_sql, $need_hash)
{
  global $page, $conf;

  $page['items'] = array();
  
  if (pwg_get_session_var('image_order',0) > 0)
  {
    $orders = get_category_preferred_image_orders();

    $conf['order_by'] = str_replace(
      'ORDER BY ',
      'ORDER BY '.$orders[ pwg_get_session_var('image_order',0) ][1].',',
      $conf['order_by']
      );
    $page['super_order_by'] = true;
  }

  $func = $need_hash ? create_function( '$q', 'return hash_from_query($q, "id");' ) : 'marray_from_query';
  
  if ('categories' == $page['section'])
  {
    if ( isset($page['flat']) or isset($page['category']) )
    {
      if (isset($page['flat']))
      {
        if ( isset($page['category']) )
        {
          $subcat_ids = get_subcat_ids( array($page['category']['id']) );
          $where_sql .= ' AND category_id IN ('.implode(',',$subcat_ids).')';
        }
      }
      else
      {
        $where_sql .= ' AND category_id = '.$page['category']['id'];
        $page['title'] = get_cat_display_name( array($page['category']), '', false);
      }

      if ( isset($page['category']) and isset($page['category']['image_order']) )
        $conf['order_by'] = 'ORDER BY '.$page['category']['image_order'];

      $query='
SELECT '.$img_fields.'
  FROM '.IMAGES_TABLE.' i INNER JOIN '.IMAGE_CATEGORY_TABLE.' ON i.id=image_id
  WHERE '.$where_sql.'
  GROUP BY i.id
  '. $conf['order_by'];

      $page['items'] = $func($query);
    }
    if ( isset($page['category']) )
    {
      $page['title'] = $page['category']['name'];
      $page['comment'] = $page['category']['comment'];
    }
    else
      $page['title'] = $conf['gallery_title'];
  }
  else if ('tags' == $page['section'])
  {
    $items = get_image_ids_for_tags( array($page['tags'][0]['id']) );
    if ( !empty($items) )
    {
      $query = '
SELECT '.$img_fields.'
  FROM '.IMAGE_CATEGORY_TABLE.' INNER JOIN '.IMAGES_TABLE.' i ON i.id=image_id
  WHERE '.$where_sql.'
    AND image_id IN ('.implode(',', $items).')
  GROUP BY i.id
  '.$conf['order_by'];

      $page['items'] = $func($query);
    }
    $page['title'] = strip_tags( get_tags_content_title() );
  }
  elseif ('search' == $page['section'])
  {
    include_once( PHPWG_ROOT_PATH .'include/functions_search.inc.php' );
    $search_result = get_search_results($page['search']);
    if ( !empty($search_result['items']) )
    {
      $query = '
SELECT '.$img_fields.'
  FROM '.IMAGES_TABLE.' i
    INNER JOIN '.IMAGE_CATEGORY_TABLE.' AS ic ON i.id = ic.image_id
  WHERE '.$where_sql.'
    AND image_id IN ('.implode(',', $search_result['items']).')
  GROUP BY i.id
  '.$conf['order_by'].'
;';
      if ( isset($search_result['as_is']) )
      {
        $page['items'] = hash_from_query($query, 'id' );

        global $item_ranks;
        $item_ranks = array_flip($search_result['items']);
function cmp_item_hash($a, $b)
{
  global $item_ranks;
  return $item_ranks [ $a['id'] ] - $item_ranks [ $b['id'] ];
}
        uasort( $page['items'], 'cmp_item_hash' );
        unset( $item_ranks );
      }
      else
        $page['items'] = $func($query);
    }

    $page['title'] = l10n('search_result');
  }
  elseif ('recent_pics' == $page['section'])
  {
    $page['super_order_by'] = true;
    $conf['order_by'] = ' ORDER BY hit DESC, file ASC';

    $query ='
SELECT '.$img_fields.'
  FROM '.IMAGES_TABLE.' i
    INNER JOIN '.IMAGE_CATEGORY_TABLE.' AS ic ON id = ic.image_id
  WHERE date_available >= SUBDATE(
      CURRENT_DATE,INTERVAL '.$user['recent_period'].' DAY)
    AND '.$where_sql.'
  GROUP BY i.id
  '.$conf['order_by'].'
  LIMIT 0, '.$conf['top_number'].'
;';

    $page['items'] = $func($query);
    $page['title'] = l10n('recent_pics_cat');
  }
  else if ('list'==$page['section'])
  {
    $query ='
SELECT '.$img_fields.'
  FROM '.IMAGES_TABLE.' i
    INNER JOIN '.IMAGE_CATEGORY_TABLE.' AS ic ON id = ic.image_id
  WHERE image_id IN ('.implode(',', $page['list']).')
    AND '.$where_sql.'
  GROUP BY i.id
  '.$conf['order_by'].'
;';

    $page['items'] = $func($query);
    $page['title'] = l10n('random_cat');
  }
  else
    die( 'section '.$page['section']. ' not handled '. __FILE__);
}

?>